SystemOrganization addCategory: #'Zinc-Seaside'!

WAServerAdaptor subclass: #ZnZincServerAdaptor
	instanceVariableNames: 'server'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Seaside'!

!ZnZincServerAdaptor commentStamp: '<historical>' prior: 0!
I convert between Seaside and Zinc HTTP requests and responses.
I am a WAServerAdaptor.

Instance Variables:
	server	<ZnServer>
		
I can be started like any other Seaside adaptor:

	ZnZincServerAdaptor startOn: 8080
	
I work together with a ZnSeasideServerAdaptorDelegate. I offer several advanced configuration options for specialized applications in my 'initialize-release' protocol. See also the 'example' class protocol of ZnSeasideServerAdaptorDelegate. !

!ZnZincServerAdaptor methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/5/2013 16:52'!
basicStart
	self server start! !

!ZnZincServerAdaptor methodsFor: 'private' stamp: 'SvenVanCaekenberghe 7/2/2012 13:45'!
basicStop
	server 
		ifNotNil: [ 
			server stop.
			server := nil ]! !

!ZnZincServerAdaptor methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 1/5/2013 16:50'!
configureDelegate
	"Set the main delegate of my server to the default one."
	
	server delegate: self defaultDelegate! !

!ZnZincServerAdaptor methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 1/5/2013 16:49'!
configureServerForBinaryReading
	"Seaside wants to do its own text conversions"

	server reader: [ :stream | ZnRequest readBinaryFrom: stream ]! !

!ZnZincServerAdaptor methodsFor: 'private' stamp: 'pmm 7/9/2014 19:39'!
convertMultipart: entity
	| dictionary |
	dictionary := ZnMultiValueDictionary new.
	entity partsDo: [ :each |
		| value |
		value := each fileName isEmptyOrNil 
			ifTrue: [ 
				each hasEntity
					ifTrue: [ self codec url decode: each fieldValueString ]
					ifFalse: [ String new ] ] 
			ifFalse: [ self convertMultipartFileField: each ] .
		dictionary at: each fieldName add: value ].
	^ dictionary 
		! !

!ZnZincServerAdaptor methodsFor: 'private' stamp: 'SvenVanCaekenberghe 2/27/2011 20:25'!
convertMultipartFileField: part  
	| file |
	(file := WAFile new)
		fileName: part fileName;
		contentType: part contentType printString;
		contents: part contents.
	^ file! !

!ZnZincServerAdaptor methodsFor: 'defaults' stamp: 'pmm 9/8/2010 16:54'!
defaultCodec
	^ GRCodec forEncoding: 'utf-8'! !

!ZnZincServerAdaptor methodsFor: 'defaults' stamp: 'SvenVanCaekenberghe 1/5/2013 16:40'!
defaultDelegate
	^ ZnSeasideServerAdaptorDelegate with: self! !

!ZnZincServerAdaptor methodsFor: 'defaults' stamp: 'SvenVanCaekenberghe 10/18/2011 14:07'!
defaultZnServer
	^ ZnServer on: self port! !

!ZnZincServerAdaptor methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 10/18/2011 14:08'!
isRunning
	^ server notNil and: [ server isRunning ]! !

!ZnZincServerAdaptor methodsFor: 'testing' stamp: 'pmm 9/8/2010 16:51'!
isStarting
	^ false ! !

!ZnZincServerAdaptor methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 10/18/2011 14:08'!
isStopped
	^ server isNil or: [ server isRunning not ]! !

!ZnZincServerAdaptor methodsFor: 'testing' stamp: 'pmm 9/8/2010 16:52'!
isStopping
	^ false ! !

!ZnZincServerAdaptor methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 1/5/2013 16:21'!
printOn: stream
	super printOn: stream.
	stream
		nextPut: $(;
		<< self statusString;
		nextPut: $)! !

!ZnZincServerAdaptor methodsFor: 'converting' stamp: 'pmm 9/9/2010 22:08'!
requestAddressFor: aZincRequest
	^ aZincRequest  headers
		at: ZnConstants remoteAddressHeader 
		ifAbsent: [ nil ]! !

!ZnZincServerAdaptor methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 6/4/2013 14:57'!
requestBodyFor: aZincRequest
	^ (aZincRequest method ~= #TRACE 
		and: [ aZincRequest hasEntity
			and: [ aZincRequest entity isEmpty not
				and: [ (aZincRequest entity contentType matches: ZnMimeType applicationFormUrlEncoded) not
					and: [ (aZincRequest entity contentType matches: ZnMimeType multiPartFormData) not ] ] ] ])
			ifTrue: [
				"Seaside wants to do its own text conversions"
				aZincRequest entity bytes asString ]
			ifFalse: [
				String new ]! !

!ZnZincServerAdaptor methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 2/27/2011 20:23'!
requestCookiesFor: aZincRequest
	| cookieString |
	cookieString := aZincRequest headers
		at: 'Cookie' ifAbsent: [ ^ #() ].
	^ WARequestCookie fromString: cookieString! !

!ZnZincServerAdaptor methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 6/4/2013 14:36'!
requestFieldsFor: aZincRequest
	| fields |
	fields := WARequestFields new.
	(aZincRequest method = #POST and: [ aZincRequest hasEntity ])
		ifTrue: [ 
			(aZincRequest entity contentType matches: ZnMimeType applicationFormUrlEncoded)
				ifTrue: [
					fields addAll: aZincRequest entity fields ].
			(aZincRequest entity contentType matches: ZnMimeType multiPartFormData)
				ifTrue: [
					fields addAll: (self convertMultipart: aZincRequest entity) ] ].
	^ fields! !

!ZnZincServerAdaptor methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 9/12/2010 10:58'!
requestHeadersFor: aZincRequest
	| fields |
	fields := Dictionary new.
	aZincRequest headersDo: [ :key :value |
		fields at: key asLowercase put: value ].
	^ fields! !

!ZnZincServerAdaptor methodsFor: 'converting' stamp: 'pmm 9/8/2010 17:15'!
requestMethodFor: aZincRequest
	^ aZincRequest method greaseString! !

!ZnZincServerAdaptor methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 2/15/2014 00:18'!
requestUrlFor: aZincRequest
	| zincUrl seasideUrl |
	zincUrl := aZincRequest url.
	(seasideUrl := WAUrl new)
		scheme: zincUrl scheme greaseString;
		host: zincUrl host;
		port: zincUrl port;
		fragment: zincUrl fragment.
	zincUrl hasPath ifTrue: [ 
		zincUrl pathSegments do: [ :each |
			each = #/ ifFalse: [ seasideUrl addToPath: each ] ].
		seasideUrl slash: zincUrl isDirectoryPath ].
	zincUrl queryDo: [ :key :value |
		seasideUrl addField: key value: value ].
	^ seasideUrl! !

!ZnZincServerAdaptor methodsFor: 'converting' stamp: 'pmm 9/8/2010 17:19'!
requestVersionFor: aZincRequest
	^ WAHttpVersion fromString: aZincRequest requestLine version! !

!ZnZincServerAdaptor methodsFor: 'converting' stamp: 'pmm 4/27/2011 18:59'!
responseFrom: aRequestContext
	| partialHeaders cookies fullHeaders seasideResponse contents entity contentType |
	seasideResponse := aRequestContext response.
	partialHeaders := seasideResponse headers.
	cookies := seasideResponse cookies.
	fullHeaders := ZnHeaders defaultResponseHeaders.
	partialHeaders keysAndValuesDo: [ :key :value |
		fullHeaders at: key put: value ].
	cookies do: [ :each |
		fullHeaders at: 'Set-Cookie' add: each oldNetscapeString.
		fullHeaders at: 'Set-Cookie2' add: each rfc2965String ].
	contentType := seasideResponse contentType greaseString.
	contents := seasideResponse contents.
	entity := (ZnEntity bytes: contents) contentType: contentType; yourself.
	^ ZnResponse new
		statusLine: (ZnStatusLine code: seasideResponse status);
		headers: fullHeaders;
		entity: entity;
		yourself! !

!ZnZincServerAdaptor methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/5/2013 16:48'!
server
	"Return the ZnServer running me, create a default one if missing and confugure it."

	server
		ifNil: [ 
			server := self defaultZnServer.
			self
				configureDelegate;
				configureServerForBinaryReading ].
	^ server! !

!ZnZincServerAdaptor methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 1/5/2013 21:34'!
server: znServer
	"I initialize the actual ZnServer instance that I will be using to znServer, 
	to prevent a fallback to #defaultZnServer"
	
	server := znServer.
	port := znServer port! !

!ZnZincServerAdaptor methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 10/18/2011 15:06'!
serverClass: znServerClass
	"I initialize the actual ZnServer instance that I will be using, 
	by instanciating znServerClass using my port"
	
	self server: (znServerClass on: self port)! !

!ZnZincServerAdaptor methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/5/2013 17:29'!
shutDown
	self isRunning
		ifTrue: [ server stop ]! !

!ZnZincServerAdaptor methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/5/2013 17:32'!
startUp
	self isRunning
		ifFalse: [ server ifNotNil: [ server start ] ]! !

!ZnZincServerAdaptor methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/9/2010 20:38'!
statusString
	^ 'zinc on port ', self port greaseString, 
		( self isRunning ifTrue: [ ' [running]' ] ifFalse: [ ' [stopped]' ] )! !

ZnZincServerAdaptor subclass: #ZnZincStaticServerAdaptor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Seaside'!

!ZnZincStaticServerAdaptor commentStamp: 'PaulDeBruicker 5/11/2012 15:21' prior: 0!
This adaptor first checks to see if the Seaside response machinery creates a succesful response and if not checks to see if the request can be served by an instance of the ZnStaticFileServerDelegate 

Start it with something like: 


	ZnZincStaticServerAdaptor startOn: 8080 andServeFilesFrom: '/var/www/'.
	

If you want to set cache expiration headers for your static assets you can run this:

	ZnZincStaticServerAdaptor default useDefaultExpirationHeaders.

after you start the ZnZincStaticServerAdaptor.

If you want to set your own expiration headers you can create a Dictionary like the one in ZnStaticFileServerDelegate class>>#defaultMimeTypeExpirations and then run:

	ZnZincStaticServerAdaptor default useCustomExpirationHeaders: myExpriationHeadersDirectory.
	
where 'myExpirationHeadersDirectory' is structured similarly to the #defaultMimeTypeExpirations !

!ZnZincStaticServerAdaptor class methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 4/20/2012 16:21'!
startOn: aPortNumber andServeFilesFrom: aDirectoryPathString
	self startOn: aPortNumber.
	self default server delegate serveFilesFrom: aDirectoryPathString! !

!ZnZincStaticServerAdaptor methodsFor: 'defaults' stamp: 'SvenVanCaekenberghe 1/5/2013 16:40'!
defaultDelegate
	^ ZnSeasideStaticServerAdaptorDelegate with: self! !

!ZnZincStaticServerAdaptor methodsFor: 'accessing' stamp: 'PaulDeBruicker 6/5/2012 21:55'!
useCustomExpirationHeaders: aDictionaryOfMimeTypesAndHeaders
	self server delegate staticDelegate mimeTypeExpirations: aDictionaryOfMimeTypesAndHeaders! !

!ZnZincStaticServerAdaptor methodsFor: 'accessing' stamp: 'PaulDeBruicker 5/17/2012 06:43'!
useDefaultExpirationHeaders
	self server delegate staticDelegate mimeTypeExpirations: ZnStaticFileServerDelegate defaultMimeTypeExpirations! !

ZnZincServerAdaptor subclass: #ZnZincStreamingServerAdaptor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Seaside'!

!ZnZincStreamingServerAdaptor commentStamp: '<historical>' prior: 0!
I convert requests and responses between Zinc and Seaside, but I do it using Seaside's WAComboResponse in order to allow data to be streamed to the client directly.!

!ZnZincStreamingServerAdaptor methodsFor: 'processing' stamp: 'KenTreis 1/11/2013 15:22'!
process: aZnRequest
	^ ZnDeferredResponse
		onWrite: [ :stream | 
			| request |
			request := ZnSeasideRequest 
				nativeRequest: aZnRequest
				stream: stream.
			super process: request ]! !

!ZnZincStreamingServerAdaptor methodsFor: 'converting' stamp: 'KenTreis 1/11/2013 15:24'!
requestFor: aZnSeasideRequest
	^super requestFor: aZnSeasideRequest nativeRequest! !

!ZnZincStreamingServerAdaptor methodsFor: 'converting' stamp: 'KenTreis 1/11/2013 15:22'!
responseFor: aZnRequest
	| bufferedStream codecStream |
	bufferedStream := GRPlatform current writeCharacterStreamOn: (String new: 4096).
	codecStream := self codec encoderFor: bufferedStream. 
	^ WAComboResponse
		onBuffered: (GRCountingStream on: codecStream)
		external: aZnRequest stream
! !

!ZnZincStreamingServerAdaptor methodsFor: 'converting' stamp: 'KenTreis 1/11/2013 15:23'!
responseFrom: aRequestContext
	"Since the response knows how to write itself on the stream (and may already be partly written),
	there's not much left to do."
	
	aRequestContext response
		flush;
		close! !

ZnResponse subclass: #ZnDeferredResponse
	instanceVariableNames: 'onWrite'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Seaside'!

!ZnDeferredResponse commentStamp: '<historical>' prior: 0!
I am a lazy ZnResponse that:

1. Does no work except to delegate to a smarter object; and
2. Doesn't even delegate until I'm told it's time to write something out on the stream.

See, I'm even lazy about being lazy. I can get away with this wanton slothfulness because of WAComboResponse's ability to handle everything (status line, headers, chunking).!

!ZnDeferredResponse class methodsFor: 'instance creation' stamp: 'KenTreis 1/10/2013 16:51'!
onWrite: aBlock
	^self new onWrite: aBlock! !

!ZnDeferredResponse methodsFor: 'initialize-release' stamp: 'KenTreis 1/10/2013 16:50'!
onWrite: aBlock
	onWrite := aBlock! !

!ZnDeferredResponse methodsFor: 'writing' stamp: 'KenTreis 1/10/2013 17:26'!
writeOn: stream
	onWrite value: stream! !

Object subclass: #ZnSeasideRequest
	instanceVariableNames: 'nativeRequest stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Seaside'!

!ZnSeasideRequest commentStamp: '<historical>' prior: 0!
I group a ZnRequest together with its stream, purely for convenience.!

!ZnSeasideRequest class methodsFor: 'instance creation' stamp: 'KenTreis 1/10/2013 21:33'!
nativeRequest: aZnRequest stream: aStream
	^ self new
		nativeRequest: aZnRequest;
		stream: aStream! !

!ZnSeasideRequest methodsFor: 'accessing' stamp: 'KenTreis 1/10/2013 21:14'!
nativeRequest
	^ nativeRequest! !

!ZnSeasideRequest methodsFor: 'accessing' stamp: 'KenTreis 1/10/2013 21:14'!
nativeRequest: anObject
	nativeRequest := anObject! !

!ZnSeasideRequest methodsFor: 'accessing' stamp: 'KenTreis 1/10/2013 21:14'!
stream
	^ stream! !

!ZnSeasideRequest methodsFor: 'accessing' stamp: 'KenTreis 1/10/2013 21:14'!
stream: anObject
	stream := anObject! !

Object subclass: #ZnSeasideServerAdaptorDelegate
	instanceVariableNames: 'adaptor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Seaside'!

!ZnSeasideServerAdaptorDelegate commentStamp: 'SvenVanCaekenberghe 9/30/2010 18:13' prior: 0!
I am ZnSeasideServerAdaptorDelegate.

I handle Zinc HTTP Server delegate's #handleRequest: using WAServerAdaptor>>#process:

Instance Variables:
	adaptor	<WAServerAdaptor>!

!ZnSeasideServerAdaptorDelegate class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 1/5/2013 21:36'!
forServer: znServer
	| adaptor |
	adaptor := ZnZincServerAdaptor new.
	adaptor unregister.
	adaptor server: znServer.
	^ self with: adaptor ! !

!ZnSeasideServerAdaptorDelegate class methodsFor: 'examples' stamp: 'SvenVanCaekenberghe 1/5/2013 21:33'!
installInDefaultServer
	^ self installInServer: ZnServer default! !

!ZnSeasideServerAdaptorDelegate class methodsFor: 'examples' stamp: 'SvenVanCaekenberghe 1/5/2013 21:33'!
installInServer: znServer
	^ self installInServer: znServer underPrefix: 'seaside'! !

!ZnSeasideServerAdaptorDelegate class methodsFor: 'examples' stamp: 'SvenVanCaekenberghe 1/7/2013 12:47'!
installInServer: znServer underPrefix: prefix
	"Install a ZnSeasideServerAdaptorDelegate in znServer under prefix,
	provided znServer contains a ZnDefaultServerDelegate instance as primary delegate.
	Note that the incoming request is destructively modified.
	Note that a global #serverPath is added to the default Seaside dispatcher."

	| seasideDelegate |
	seasideDelegate := self forServer: znServer.
	znServer delegate
		map: 'seaside'
		to: [ :request | 
			request uri removeFirstPathSegment.
			seasideDelegate handleRequest: request ].
	WAAdmin defaultDispatcher serverPath: prefix.
	^ seasideDelegate! !

!ZnSeasideServerAdaptorDelegate class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/30/2010 17:14'!
with: aSeasideServerAdaptor
	^ self new
		adaptor: aSeasideServerAdaptor;
		yourself! !

!ZnSeasideServerAdaptorDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 17:13'!
adaptor
	^ adaptor ! !

!ZnSeasideServerAdaptorDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 17:12'!
adaptor: aSeasideServerAdaptor
	adaptor := aSeasideServerAdaptor ! !

!ZnSeasideServerAdaptorDelegate methodsFor: 'public' stamp: 'PaulDeBruicker 6/5/2012 21:52'!
handleRequest: znRequest
	"I pass a Zinc HTTP Components ZnRequest to my Seaside Server Adaptor using #process:"

	^ self adaptor process: znRequest! !

!ZnSeasideServerAdaptorDelegate methodsFor: 'public' stamp: 'SvenVanCaekenberghe 1/5/2013 21:09'!
value: request
	"I implement the generic #value: message as equivalent to #handleRequest:"
	
	^ self handleRequest: request! !

ZnSeasideServerAdaptorDelegate subclass: #ZnSeasideStaticServerAdaptorDelegate
	instanceVariableNames: 'staticDelegate wwwRootDirectory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Seaside'!

!ZnSeasideStaticServerAdaptorDelegate commentStamp: '<historical>' prior: 0!
This adaptor first checks to see if the Seaside response machinery creates a succesful response and if not checks to see if the request can be served by an instance of the ZnStaticFileServerDelegate 

Instance Variables:
    staticDelegate	aZnStaticFileServerDelegate
    wwwRootDirectory - The directory on your file system where the JS/CSS/JPEG etc. files you want to serve reside!

!ZnSeasideStaticServerAdaptorDelegate methodsFor: 'public' stamp: 'SeanDeNigris 2/12/2013 15:23'!
handleRequest: znRequest
	"If the Seaside request processing apparatus returns an HTTP response with an error 
	check the filesystem for a file that matches the uri of the request, if found send the file if not send an error."

	| response |
	response := self adaptor process: znRequest.
	^ (response isError and: [ response isAuthenticationRequired not ])
			ifTrue: [ self staticDelegate handleRequest: znRequest ]
			ifFalse: [ response ].! !

!ZnSeasideStaticServerAdaptorDelegate methodsFor: 'initialization' stamp: 'PaulDeBruicker 4/19/2012 16:28'!
initializeStaticDelegate
	^ ZnStaticFileServerDelegate new
		directory: self wwwRootDirectory;
		yourself! !

!ZnSeasideStaticServerAdaptorDelegate methodsFor: 'updating' stamp: 'SvenVanCaekenberghe 8/28/2012 21:18'!
serveFilesFrom: aDirectoryPathString
	self wwwRootDirectory: (ZnFileSystemUtils directory: aDirectoryPathString )! !

!ZnSeasideStaticServerAdaptorDelegate methodsFor: 'accessing' stamp: 'PaulDeBruicker 4/19/2012 16:26'!
staticDelegate
	^ staticDelegate ifNil: [ staticDelegate := self initializeStaticDelegate ]! !

!ZnSeasideStaticServerAdaptorDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/28/2012 21:19'!
wwwRootDirectory
	"This is the root directory for your css/js/images etc...."

	^ wwwRootDirectory ifNil: [ 
		wwwRootDirectory := ZnFileSystemUtils directory: '/var/www' ]! !

!ZnSeasideStaticServerAdaptorDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/20/2012 10:53'!
wwwRootDirectory: fileDirectory
	"This is the root directory for your css/js/images etc...."

	wwwRootDirectory := fileDirectory! !
